﻿# coding=utf-8

"""检查结果返回值"""
CHECK_PASS = 0
CHECK_FAIL = 1

import re
import time

CLI = None
LANGUAGE = None
IP_ADDR = None


def execute(context):
    """
    Function name      : getdev_type
    Function describe  : 添加主机获取信息
    Input              : context
    Return             : flag, cliRet, errMsg
    """

    # 给全局变量赋初始值，按照要求传递上下文进去。
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = "en"
    global IP_ADDR
    IP_ADDR = context.get("dev").getIp()

    name = check_and_get_result(get_device_name())
    if "VMware".lower() in name.lower():
        name = name
        dev_type = check_and_get_result(get_dev_type())
        sn = check_and_get_result(get_serial_number())
        version = check_and_get_result(get_product_version())
    elif name=="Linux":
        dev_type = CLI.execCmdWithTimout("cat /etc/*-release", 60)
        if dev_type.find('VMware') >= 0:
            dev_type = check_and_get_result(get_dev_type())
            sn = check_and_get_result(get_serial_number_by_linux())
            version = check_and_get_result(get_product_version())
            name = "VMware";
        else:
            dev_type=""
            sn=""
            version=""
    else:
        name = "not VMware"
        dev_type=""
        sn=""
        version=""

    return {"name": name, "type":dev_type, "sn": sn, "version": version}


# 取中间一个有效值
def check_and_get_result((fun_flag, result, fun_err_msg)):
    """

    :rtype : object
    """
    return str(result)


def get_dev_type():
    """
    Function name      : getdev_type
    Function describe  : 获取操作系统类型
    Input              : 
    Return             : funFlag, dev_type, funErrMsg
    """
    fun_flag = CHECK_FAIL
    dev_type = ''
    fun_err_msg = ''
    op_sys_info = CLI.execCmdWithTimout("vmware -v", 60)
    dev_type = op_sys_info.splitlines()[1].split()[1].strip()+" "+op_sys_info.splitlines()[1].split()[2].strip()
    if '' != dev_type:
        fun_flag = CHECK_PASS
    return fun_flag, dev_type, fun_err_msg


# need to explain this function.
def fetch_ubuntu_base_info():
    cli_ret = CLI.execCmdWithTimout("cat /etc/issue", 60)
    cli_ret_list = cli_ret.splitlines()
    cli_ret_lists = cli_ret_list[1].split(" ")
    return cli_ret_lists[0] + cli_ret_lists[1]


# need to explain this function.
def get_device_name():
    """
    Function name      : getdev_type
    Function describe  : 获取主机名
    Input              : 
    Return             : fun_flag, deviceName, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    cli_ret = CLI.execCmdWithTimout("uname -a", 60)
    device_name = cli_ret.splitlines()[1].split()[0]
    if "VMkernel" == device_name:
        device_name = "VMware"

    # here
    if '' == device_name:
        if LANGUAGE == "zh":
            fun_err_msg = u"获取主机名称失败。"
        else:
            fun_err_msg = "Get hostname failed."
    else:
        fun_flag = CHECK_PASS
    return fun_flag, device_name, fun_err_msg


# need to explain this function.
def get_product_version():
    """
    Function name      : getdev_type
    Function describe  : 获取操作系统版本号
    Input              : 
    Return             : cliFlag, productVersion, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    dev_type = ''
    fun_err_msg = ''
    op_sys_info = CLI.execCmdWithTimout("vmware -v", 60)
    product_version = op_sys_info.splitlines()[1].split()[2].strip()
    if '' != product_version:
        fun_flag = CHECK_PASS
    return fun_flag, product_version, fun_err_msg


#need to explain this function.
def get_serial_number():
    """
    Function name      : getdev_type
    Function describe  : 获取主机SN号
    Input              : ipStr
    Return             : fun_flag, serialNumber, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    serial_number = ''
    mac_str = ""
    mac_address = CLI.execCmdWithTimout("esxcfg-vmknic -l|grep " + IP_ADDR + "|awk '{print $8}'", 60)
    mac_arr = mac_address.splitlines()[1].strip().split(':');
    for mac in mac_arr:
        if(len(mac)==1):
            mac_str="0"
        else:
            mac_str+=mac
    if "0"==mac_str:
        serial_number = get_time_str()+IP_ADDR
    else:
        serial_number = mac_str + IP_ADDR
        
    if '' != serial_number:
        fun_flag = CHECK_PASS
    else:
        serial_number = "SN" + get_time_str()+IP_ADDR
    serial_number = serial_number.replace('.', '')
        
    return fun_flag, serial_number, fun_err_msg

def get_time_str():
    t = time.strftime('%Y-%m-%d-%H-%M-%S',time.localtime(time.time()))
    t = t.replace('-', '')
    return t
 
#need to explain this function.
def get_serial_number_by_linux():
    """
    Function name      : getdev_type
    Function describe  : 获取主机SN号
    Input              : ipStr
    Return             : fun_flag, serialNumber, fun_err_msg
    """
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    serial_number = ''
    mac_str=""
    if_config_str = CLI.execCmdWithTimout("/sbin/ifconfig", 60)
    ip_link_show = CLI.execCmdWithTimout("/sbin/ip link show", 60)
    
    if ip_link_show.find("TOOLKIT_EXE_CMD_FAILED") >0:
        ip_link_show = CLI.execCmdWithTimout("ip link show", 60)

    temp_list = if_config_str.splitlines()
    temp_list_2 = ip_link_show.splitlines()

    for i in range(0, len(temp_list)):
        if IP_ADDR in temp_list[i]:
            eth_name = temp_list[i - 1].split()[0]
            if eth_name.find(":")>=0:
                eth_name = eth_name.split(':')[0]

            eth_name = eth_name.split(':')[0]
            for i in range(0, len(temp_list_2)):
                if eth_name in temp_list_2[i]:
                    mac_address = temp_list_2[i+1].split()[1]
                    break
            mac_arr = mac_address.strip().split(':');

            for mac in mac_arr:
                if(len(mac)==1):
                    mac_str="0"
                else:
                    mac_str+=mac
            if "0"==mac_str:
                serial_number = get_time_str()+IP_ADDR
            else:
                serial_number = mac_str + IP_ADDR
            fun_flag = CHECK_PASS
            break
    if ''==serial_number:
        serial_number = "SN" + get_time_str()+IP_ADDR
        if LANGUAGE == "zh":
            fun_err_msg = u"获取设备版本失败。"
        else:
            fun_err_msg = "Get product version failed."
    
    serial_number = serial_number.replace('.', '')
    
    return fun_flag, serial_number, fun_err_msg
