# -*- coding: UTF-8 -*-
"""检查结果返回值"""
CHECK_PASS = 0
CHECK_FAIL = 1

import time
import re

CLI = None
LANGUAGE = None
IP_ADDR = None

#交换机型号
SWITCH_TYPE_DICT = {
                    "1":"Brocade 1000 Switches",
                    "2":"Brocade 2800 Switch",
                    "3":"Brocade 2100/2400 Switches",
                    "4":"Brocade 20x0 Switches",
                    "5":"Brocade 22x0 Switches",
                    "6":"Brocade 2800 Switch",
                    "7":"Brocade 2000 Switch",
                    "9":"Brocade 3800 Switch",
                    "10":"Brocade 12000 Director",
                    "12":"Brocade 3900 Switch",
                    "16":"Brocade 3200 Switch",
                    "17":"Brocade 3800VL",
                    "18":"Brocade 3000 Switch",
                    "21":"Brocade 24000 Director",
                    "22":"Brocade 3016 Switch",
                    "26":"Brocade 3850 Switch",
                    "27":"Brocade 3250 Switch",
                    "29":"Brocade 4012 Embedded Switch",
                    "32":"Brocade 4100 Switch",
                    "33":"Brocade 3014 Switch",
                    "34":"Brocade 200E Switch",
                    "37":"Brocade 4020 Embedded Switch",
                    "38":"Brocade 7420 Switch",
                    "40":"Brocade FCR Front Domain",
                    "41":"Brocade FCR Xlate Domain",
                    "42":"Brocade 48000 Director",
                    "43":"Brocade 4024 Embedded Switch",
                    "44":"Brocade 4900 Switch",
                    "45":"Brocade 4016 Embedded Switch",
                    "46":"Brocade 7500 Switch",
                    "51":"Brocade 4018 Embedded Switch",
                    "55":"Brocade 7600 Switch",
                    "58":"Brocade 5000 Switch",
                    "61":"Brocade 4424 Embedded Switch",
                    "62":"Brocade DCX Backbone",
                    "64":"Brocade 5300 Switch",
                    "66":"Brocade 5100 Switch",
                    "67":"Brocade Encryption Switch",
                    "69":"Brocade 5410 Switch",
                    "70":"Brocade 5410 Embedded Switch",
                    "71":"Brocade 300 Switch",
                    "72":"Brocade 5480 Embedded Switch",
                    "73":"Brocade 5470 Embedded Switch",
                    "75":"Brocade M5424 Embedded Switch",
                    "76":"Brocade 8000 Switch",
                    "77":"Brocade DCX-4S Switch",
                    "83":"Brocade 7800 Extension Switch",
                    "86":"Brocade 5450 Embedded Switch",
                    "87":"Brocade 5460 Embedded Switch",
                    "90":"Brocade 8470 Embedded Switch",
                    "92":"Brocade VA-40FC Switch",
                    "95":"Brocade VDX 6720-24 DC Switch",
                    "96":"Brocade VDX 6730-32 DC Switch",
                    "97":"Brocade VDX 6720-60 DC Switch",
                    "98":"Brocade VDX 6730-76 DC Switch",
                    "108":"Dell M8428-k FCoE Embedded Switch",
                    "109":"Brocade 6510 Switch",
                    "116":"Brocade VDX 6710 DC Switch",
                    "117":"Brocade 6547 Embedded Switch",
                    "118":"Brocade 6505 Switch",
                    "120":"Brocade DCX 8510-8 Backbone",
                    "121":"Brocade DCX 8510-4 Backbone"
}

def execute(context):
    """
    Function name      : getdev_type
    Function describe  : 添加主机获取信息
    Input              : context
    Return             : flag, cliRet, errMsg
    """

    # 给全局变量赋初始值，按照要求传递上下文进去。
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = "en"
    global IP_ADDR
    IP_ADDR = context.get("dev").getIp()
    name = check_and_get_result(getSwitchType())
    if ('brocade' in name.lower()) or ('dell' in name.lower()):
        name = 'Brocade'
    else:
        name = 'Not Brocade'

    dev_type = check_and_get_result(getSwitchType())
    sn = check_and_get_result(getSwitchWWN())
    version = check_and_get_result(getSwitchVersion())

    return {"name": name, "type":dev_type, "sn": sn, "version": version}

# 取中间一个有效值
def check_and_get_result((fun_flag, result, fun_err_msg)):
    """

    :rtype : object
    """
    return str(result)
    
def getSwitchType():
    '''
    @summary: 获取交换机类型
    @param cli: cli对象
    @param lang: 语言lang
    @return: (falg, ret, errMsg)
        flag: 
            True： 判断时正常
            False： 判断时异常
        ret: 
            flag为True时，交换机类型
            flag为False时，返回cli回显
        errMsg: 错误时的消息
    '''   
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    cli_ret = CLI.execCmdNoLogTimout("switchshow", 60)
    cli_ret_list = cli_ret.splitlines()
    
    switchType = ""
    for line in cli_ret_list:
        fields = line.split(":")
        if len(fields) < 2:
            continue
        
        fieldName = fields[0].strip()
        fieldValue = fields[1].strip()
        
        if fieldName == "switchType":
            switchTypeInfo = fieldValue
            switchTypeNum = switchTypeInfo.split(".")[0]
            switchType = SWITCH_TYPE_DICT.get(switchTypeNum, "")
            
    # here
    if len(switchType) == 0:
        if LANGUAGE == "zh":
            fun_err_msg = u"获取交换机类型失败。"
        else:
            fun_err_msg = "Get switch type failed."
    else:
        fun_flag = CHECK_PASS
    return fun_flag, switchType, fun_err_msg


def getSwitchVersion():
    '''
    @summary: 获取交换机版本号
    @param cli: cli对象
    @param lang: 语言lang
    @return: (falg, ret, errMsg)
        flag: 
            True： 判断时正常
            False： 判断时异常
        ret: 
            flag为True时，交换机版本号
            flag为False时，返回cli回显
        errMsg: 错误时的消息
    '''   
  
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    cli_ret = CLI.execCmdNoLogTimout("switchshow", 60)
    cli_ret_list = cli_ret.splitlines()
    
    switchVersion = ""
    for line in cli_ret_list:
        fields = line.split(":")
        if len(fields) < 2:
            continue
        
        fieldName = fields[0].strip()
        fieldValue = fields[1].strip()
        
        if fieldName == "Fabric OS":
            switchVersion = fieldValue
            
    # here
    if len(switchVersion) == 0:
        if LANGUAGE == "zh":
            fun_err_msg = u"获取交换机版本号失败。"
        else:
            fun_err_msg = "Get switch version failed."
    else:
        fun_flag = CHECK_PASS
    return fun_flag, switchVersion, fun_err_msg

def getSwitchWWN():
    '''
    @summary: 获取交换机WWN号
    @param cli: cli对象
    @param lang: 语言lang
    @return: (falg, ret, errMsg)
        flag: 
            True： 判断时正常
            False： 判断时异常
        ret: 
            flag为True时，交换机WWN号
            flag为False时，返回cli回显
        errMsg: 错误时的消息
    '''   
  
    
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    cli_ret = CLI.execCmdNoLogTimout("switchshow", 60)
    cli_ret_list = cli_ret.splitlines()
    
    switchWwn = "" 
    for line in cli_ret_list:
            
        fields = line.split(":")
        if len(fields) < 2 :
            continue
        
        fieldName = fields[0].strip()      
        if fieldName == "switchWwn":
            switchWwn = line[line.index(":") + 1:].strip().replace(":","")
            
            
    # here
    if len(switchWwn) == 0:
        if LANGUAGE == "zh":
            fun_err_msg = u"获取交换机WWN号失败。"
        else:
            fun_err_msg = "Get switch WWN failed."
    else:
        fun_flag = CHECK_PASS
    return fun_flag, switchWwn, fun_err_msg
