﻿# -*- coding: UTF-8 -*-
from com.huawei.ism.tool.infograb.context import EvalResultEnum
from com.huawei.ism.tool.infograb.context import ItemEvalResult
import sys
import traceback
import constants
import re

class log():

    """
            功能：日志记录函数的子函数，用于获取调用函数和调用行号
            参数： MAX_CALLER_LEVEL：最大调用关系层数
            返回值：调用函数信息
    """
    @staticmethod
    def getCallerInfo(MAX_CALLER_LEVEL = 5, skipLastLevel = True):
        #从堆栈中获取调用函数和行号

        #初始化参数
        funcBack = sys._getframe().f_back
        if True == skipLastLevel: #忽略最近的调用关系
            funcBack = funcBack.f_back
            MAX_CALLER_LEVEL -= 1

        #生成函数调用关系
        callerInfo = ""
        for _ in range(0, MAX_CALLER_LEVEL):

            #获取该级调用函数和行号
            if hasattr(funcBack, "f_code") and hasattr(funcBack, "f_lineno"):
                funcName = funcBack.f_code.co_name
                lineNumber = funcBack.f_lineno
                callerInfo = " [" + str(funcName) + ":" + str(lineNumber) + "]" + callerInfo
            else:
                break

            #刷新Back函数
            if hasattr(funcBack, "f_back"):
                funcBack = funcBack.f_back
            else:
                break

        #返回函数调用关系
        return callerInfo

    @staticmethod
    def debug(context, info):
        """
                    功能：记录调试信息info到工具日志中
                    参数：context=工具上下文；info=要记录的信息
                    返回值：True=成功；False=失败
        """
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").debug('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def error(context, info):
        """
                    功能：记录错误信息info到工具日志中
                    参数：context=工具上下文；info=要记录的信息
                    返回值：True=成功；False=失败
        """
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").error('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def info(context, info):
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").info('[ToolLog]:' + logInfo)
            return True
        else:
            return False

    @staticmethod
    def warn(context, info):
        logInfo = info + log.getCallerInfo()
        if context and "Logger" in context:
            context.get("Logger").warn('[ToolLog]:' + logInfo)
            return True
        else:
            return False

def parseWwpn(context, wwpn):
    '''
    @summary: Parse WWPN to get MAC address and controller node ID.
    @param context: Python execution context.
    @param wwpn: WWPN of a port.
    @return: (isParseSucc, macAddr, nodeId)
    '''

    try:
        if ':' not in wwpn:
            wwpn = int(wwpn, 16)
        else:
            wwpn = int(wwpn.replace(':', ''), 16)

        ctrlNodeId = wwpn >> 52 & 0x7
        ctrlEncMac = wwpn & 0xffffffffffff
        return True, ctrlEncMac, int(ctrlNodeId)
    except Exception,e:
        log.error(context, 'Parse WWPN exception:' + str(e))
        return False, -1, -1

def qryUltrapathVersion(context):
    '''
    @summary: Query the HUAWEI ultrapath version.
    @param context: Python execution context.
    @return (isQrySucc, ultrapathVersion, cliRet)
    '''
    cli = context.get("SSH")
    if not cli:
        return False, ''

    cliRet = ''
    try:
        cliRet = cli.execCmdWithNoCheckResult(constants.QRY_ULTRAPATH_CMD,
                                       constants.HOST_CMD_TIMEOUT,
                                       constants.LINUX_CMD_DFT_END_LIST)
        verPattern = 'Software Version|UltraPath for Linux|Driver   Version'
        if re.search(verPattern, cliRet, re.I):
            for line in cliRet.splitlines():
                if re.search(verPattern, line, re.I):
                    return True, line.split(':')[1].strip(), cliRet

        return True, '', cliRet
    except:
        log.error(context, 'Query ultrapath version exception:' + traceback.format_exc())
        return False, '', cliRet
    
def qryAndSaveUltrapathVersion(context):
    '''
    @summary: 查询多路径版本并保存到HTML
    @param param: context 上下文
    '''
    isSuccess, version,cliReturn = qryUltrapathVersion(context)
    cmdDisplay = context.get("ret_map")
    cmdDisplay.put("cmd_display_multipath_version_huawei",cliReturn)
    return isSuccess, version,cliReturn
def getUltrapathIntVer(context, ver):
    '''
    @summary: Convert String version to a tuple with 3 version.
    @param context: Python execution context.
    @param ver: String version.
    @return: version tuple such as (5, 1, 17)
    '''
    try:
        return tuple(map(int, ver.split('.')))
    except:
        log.error(context, 'Convert ultrapath version exception:' + traceback.format_exc())
        return (0, 0, 0)

def qryMultipathdStatus(context):
    '''
    @summary: Query the multipathd status (Linux multipath service status).
    @param context: Python execution context.
    @return (isQrySucc, isMultipathdRunning, cliRet)
    '''
    cli = context.get("SSH")
    if not cli:
        return False, False, ''

    cliRet = ''
    endStr = ['#', '$', '>', '(press RETURN)']
    try:
        cliRet = cli.execCmdWithNoCheckResult(constants.QRY_MULTIPATHD_STATUS,
                                       constants.HOST_CMD_SHORT_TIMEOUT,
                                       endStr)
        if 'multipathd: unrecognized service' in cliRet or 'command not found' in cliRet :
            reCheckCmd = 'ps -e | grep multipathd |cat'
            cliRet = cli.execCmdWithNoCheckResult(reCheckCmd,
                                       constants.HOST_CMD_SHORT_TIMEOUT)
            irRunning = cliRet.replace(reCheckCmd, '').find('multipathd') > -1
            cmdResult = reCheckCmd + '\r\nmultipathd:'
            return True, irRunning, cmdResult + 'running' if irRunning else cmdResult + ':stopped'
        if "(press RETURN)" in cliRet:
            cliRet = cliRet + cli.execCmdWithNoCheckResult("\r",
                                       constants.HOST_CMD_SHORT_TIMEOUT,
                                       endStr)
        if "running" in cliRet.lower() or u"正在运行" in cliRet:
            return True, True, cliRet

        return True, False, cliRet
    except:
        log.error(context, 'Query multipathd service status exception:' + traceback.format_exc())
        return False, False, cliRet

def getArrayIdsManagedByUltrapath(context):
    '''
    @summary: Query the array id of all storage array managed by ultrapath.
    @param context: Python execution context.
    @return (isQrySucc, ArrayIDList, cliRet)
    '''
    cli = context.get("SSH")
    if not cli:
        return False, [], ''
    try:
        cliRet = cli.execCmdWithNoCheckResult(constants.QRY_ARRAY_ID_CMD,
                                    constants.HOST_CMD_TIMEOUT,
                                    constants.LINUX_CMD_DFT_END_LIST)
        arrayIdList = []
        isDataStart = False
        for line in cliRet.splitlines():
            if not isDataStart and re.search('Array ID', line, re.I):
                isDataStart = True
                continue

            fields = line.split()
            if isDataStart and len(fields) >= 3:
                if fields[0].isdigit():
                    arrayIdList.append(fields[0])

        return True, arrayIdList, cliRet
    except:
        log.error(context, 'Query array IDs exception:' + traceback.format_exc())
        return False, [], ''

def getVlunIdsManagedByUltrapath(context, isMiddleVer = True):
    '''
    @summary: Query the vlun id of all storage array managed by ultrapath.
    @param context: Python execution context.
    @return (isQrySucc, vlunIDList, cliRet)
    '''
    cli = context.get("SSH")
    if not cli:
        return False, [], ''
    try:
        cliRet = cli.execCmdWithNoCheckResult(constants.QRY_VLUN_ID_CMD if isMiddleVer else
                                       constants.High_VER_QRY_VLUN_ID_CMD,
                                    constants.HOST_CMD_TIMEOUT)
        vlunIdList = []
        isDataStart = False
        for line in cliRet.splitlines():
            if not isDataStart and re.search('Vlun ID', line, re.I):
                isDataStart = True
                continue

            fields = line.split()
            if isDataStart and len(fields) >= 3:
                if fields[0].isdigit():
                    vlunIdList.append(fields[0].strip())

        #De-duplicate same LUN ID for hyper metro LUN.
        return True, list(set(vlunIdList)), cliRet
    except:
        log.error(context, 'Query VLUN IDs exception:' + traceback.format_exc())
        return False, [], ''


def genEvalItemObj(itemKey, evalResult, cliRet, errMsgKey, paraList = None):
    '''
    @summary: Create an evaluate result object of Java ItemEvalResult type.
    @param evalResut: Evaluate result.
    @param errMsgKey: error message key
    @param cliRet: CLI return message.
    @param paraList:Error message parameter list of string type.
    @return: item evaluate result.
    '''
    itemEvalResult = ItemEvalResult(itemKey, evalResult)
    itemEvalResult.setCliRet(cliRet)
    itemEvalResult.setErrMsgKey(errMsgKey)
    itemEvalResult.setErrParas(paraList)
    return itemEvalResult

def genEvalItemObjNoCliRet(itemKey, evalResult,errMsgKey, paraList = None):
    '''
    @summary: Create an evaluate result object of Java ItemEvalResult type.
    @param evalResut: Evaluate result.
    @param errMsgKey: error message key
    @param paraList:Error message parameter list of string type.
    @return: item evaluate result.
    '''
    itemEvalResult = ItemEvalResult(itemKey, evalResult)
    itemEvalResult.setErrMsgKey(errMsgKey)
    itemEvalResult.setErrParas(paraList)
    return itemEvalResult
def hasRedundantPathForCtrlIdPath(context, lunPath, isNodeId = False):
    '''
    @summary: Check whether the LUN has redundant path.
    @param context: Python execution context.
    @param lunPath: LUN PATH list, such as ['0A', '0B', '1A', '1B'] or [0, 1, 2, 3]
    @return: True if redundant else False.
    '''
    if not lunPath:
        return False
    try:
        if not isNodeId:
            lunPath = list(map(str.upper, map(str, lunPath)))
            lunPath = list(set(lunPath))
            evenPth = list(filter(lambda ctrlId: ctrlId[-1] in ['A', 'C'], lunPath))
            oddPth = list(filter(lambda ctrlId: ctrlId[-1] in ['B', 'D'], lunPath))
        else:
            evenPth = list(filter(lambda ctrlId: ctrlId in [0, 2], lunPath))
            oddPth = list(filter(lambda ctrlId: ctrlId in [1, 3], lunPath))

        return len(evenPth) > 0 and len(oddPth) > 0
    except:
        log.error(context, 'check redundant path exception:' + traceback.format_exc())
        return False

def chkSingleLunPathRedundant(context, vlunPthRet):
    '''
    @summary: Check single LUN path, for migration LUN, ignore and check pass.
    @param context: Python execution context.
    @param vlunPthRet: The command result of upadmin show vlun id = < LUNID >
    @return:(isHyperMetroLUN, chkPass)
    '''

    isHyperMetorLun = False
    if re.search('Aggregation.*Type.*Hyper.*Metro.*', vlunPthRet, re.I):
        isHyperMetorLun = True
        log.debug(context, 'This is hyper metro LUN.')
        firstSubLunStartIdx = vlunPthRet.find('Aggregation Member#0')
        secondSubLunStartIdx = vlunPthRet.find('Aggregation Member#1')

    if re.search('Aggregation.*Type.*Migration.*', vlunPthRet, re.I):
        log.debug(context, 'This is a migration LUN, ignore and check pass.')
        return False, True

    if not isHyperMetorLun:
        return False, checkLunPathRedundant(context, vlunPthRet)[1]
    elif -1 == firstSubLunStartIdx or -1 == secondSubLunStartIdx:#Fast fail.
        log.error(context, 'One or two sub LUN not found, check failed.')
        return True, False
    else:
        fstSubLunArrSN, fstLunRedundant = checkLunPathRedundant(context, vlunPthRet[:secondSubLunStartIdx])
        sndSubLunArrSN, sndLunRedundant = checkLunPathRedundant(context, vlunPthRet[secondSubLunStartIdx:])
        if not fstSubLunArrSN or not sndSubLunArrSN or fstSubLunArrSN == sndSubLunArrSN:
            log.error(context, 'A pair of hyper metro LUNs are from the same array, check failed.')
            return True, False
        else:
            return True, fstLunRedundant and sndLunRedundant

def checkLunPathRedundant(context, vlunPthRet):
    '''
    @summary: 检查单个非双活LUN的路径是否冗余或者双活LUN的一个子LUN的链路是否冗余.
    @param context: Python上下文.
    @param vlunPthRet: 单个非双活LUN的路径回显或者双活LUN回显的一部分（单个子LUN回显信息).
    @return:(isChkSucc, isRedundant)
    '''
    vlunPthList = []
    arraySN, vlunPth = '', ''
    for line in vlunPthRet.splitlines():
        line = line.strip().lower()

        if 'array sn' in line:
            arraySN = line.split(':')[1].strip()

        if line.startswith('controller'):
            vlunPth = line.split()[-1].strip()

        if vlunPth and line.startswith('path'):
            vlunPthSta = line.split(':')[-1].strip()
            if vlunPthSta == 'normal':
                vlunPthList.append(vlunPth)

    return arraySN, hasRedundantPathForCtrlIdPath(context, vlunPthList)

def getTargetPortNum(context):
    '''
    @summary: 获取目标器端口编号(存储上报给目标器的）.
    @param context: Python执行上下文环境.
    @return: isQrySucc, portNumbers, cliRet:
                            portNumbers:端口编号列表，如：['0:0:0', '0:0:1']。
    '''
    cli = context.get("SSH")
    if not cli:
        return False, [], ''
    try:
        cliRet = ''
        cliRet = cli.execCmdWithNoCheckResult(constants.QRY_DEV_NAME_CMD,
                                    constants.HOST_CMD_TIMEOUT,
                                    constants.LINUX_CMD_DFT_END_LIST)
        portNums = []
        tgtPortNumPattern = re.compile(constants.TARGET_PORT_NUM_PATTERN)
        for line in cliRet.splitlines():
            line = line.strip()
            portNumMathObj = tgtPortNumPattern.search(line)
            if portNumMathObj:
                portNums.append(portNumMathObj.group())
        return True, list(set(portNums)), cliRet
    except:
        log.error(context, 'Query target port numbers exception:' + traceback.format_exc())
        return False, [], cliRet
    
def buildMultiPathVersion(hwMultiPathVer,buildInMultiPathInfo):
    '''
    @summary: 生成多路径状态信息
    @param param: hwMultiPathVer 自研多路径信息
    @param param: buildInMultiPathInfo 自带多路径信息
    '''
    result = {'HuaweiMultipathVersion': 'NA', 'SelfMultiPath': 'disabled'}
    findResult = re.findall(r'.+:\s+\d+.\d{2}.\d{3}', hwMultiPathVer, re.I)
    if len(findResult) > 0:
        result['HuaweiMultipathVersion'] = findResult[0].split(':')[-1].strip()
    for line in buildInMultiPathInfo:
        if ('正在运行' in line or 'running' in line.lower()):
            result['SelfMultiPath'] = 'enabled'
            break
    return str(result).replace("u\'", "\'")
def getPortNumPathDict(context, tgtPortNums):
    '''
    @summary: 获取目标器端口编号(存储上报给目标器的）.
    @param context: Python执行上下文环境.
    @param tgtPortNums: TGT端口编号的列表，如：['0:0:0', '0:0:1']。
    @return: 端口号到路径的映射字典:
                            如：{'0:0:0':'0A', '0:0:1':'0b'} or {'0:0:0':0, '0:0:1':1}。
    '''
    if not tgtPortNums:
        return {}, ''
    
    cli = context.get("SSH")
    if not cli:
        return {}, ''
    
    tgtPortPthMap = {}
    allCliRet = ''
    for tgtPortNum in tgtPortNums:
        try:
            qryTgtPortPthCmd = constants.QRY_TGT_PORT_PATH_CMD % tgtPortNum
            cliRet = cli.execCmdWithNoCheckResult(qryTgtPortPthCmd,
                                                  constants.HOST_CMD_TIMEOUT,
                                                  constants.LINUX_CMD_DFT_END_LIST)
            allCliRet += ('\n' + cliRet)
            for line in cliRet.splitlines():
                line = line.strip()
                if line.startswith('0x'):
                    isParseSucc, _, nodeId = parseWwpn(context, line)
                    if not isParseSucc:
                        log.error(context, 'Parse WWPN failed:' + unicode(line))
                        break
                    else:
                        tgtPortPthMap[tgtPortNum] = nodeId
        except:
            log.error(context, 'Query target port number:' + unicode(tgtPortNum)
                       + ' exception:' + traceback.format_exc())
            continue
    
    return tgtPortPthMap, allCliRet

def errorMsgHandler(errorMessage, cmdReturn, cmdDesc,LANGUAGE):
    '''
    @summary: 将错误消息进行组装
    @param param: errorMessage 命令返回
    @param param: cmdReturn CMD命令回显
    @param param: cmdDesc CMD描述
    '''
    if cmdReturn.find('command not found') > 0 or cmdReturn.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmdReturn.find('TOOLKIT_EXE_CMD_FAILED') > 0:
        if "en" == LANGUAGE:
            errorMessage += cmdDesc + ":\texecute failed\r\n"
        else:
            errorMessage += cmdDesc + u":\t执行失败\r\n"
    else:
        if "en" == LANGUAGE:
            errorMessage += cmdDesc + ":\texecute success\r\n"
        else:
            errorMessage += cmdDesc + u":\t执行成功\r\n"
    return errorMessage