# --*-- coding:utf-8 --*--
import sys 
import os
import re
reload(sys) 
sys.setdefaultencoding('utf-8')
path = os.path.dirname(os.path.abspath(__file__))
path =os.path.join(path, "..\\..\\..")
sys.path.append(path)
from common.contentParse import getSingleOrMoreCommandRetFromTxt,is_belong_lst
from common.commonLogger import getLogger

HBA_MODEL_FW_CMDS = "cmd_display_hba_card_fcmsutil_modelfw_"
HBA_DRVER_STATE_CMDS = 'cmd_display_hba_card_fcmsutil_drstate_'

Logger = getLogger()

def execute(context):
    '''
    @summary:脚本入口
    @param context:上下文
    '''
    hbaObjs = getAllHbaObjs(context)
    
    result = getAllWWPNS(hbaObjs)
    context.put('result', result)
    return context

  
 

def getAllWWPNS(hbas):
    '''
    @summary:过滤掉state不为online的HBA卡。
    @param hbas:对象列表
    @return wwpns:wwpn
    '''
    wwpns = ''
    for hba in hbas:
        if 'ONLINE'.lower() != hba.getState().lower():   
            continue
        wwpns = wwpns +';'+ hba.getHbaWWPN()
    if not wwpns:
        wwpns = 'NA'
    else:
        wwpns = wwpns[1:]
    return wwpns
        
        
def getAllHbaObjs(context):
    '''
    @summary: 获取所有的hba对象
    @param context:上下文
    @return hbaObjLst:所有hba对象列表 
    '''
    hbaObjLst = []
    
    eachHbaModelFwLst = getSingleOrMoreCommandRetFromTxt(context, HBA_MODEL_FW_CMDS,True)
    eachHbaDriStateLst = getSingleOrMoreCommandRetFromTxt(context, HBA_DRVER_STATE_CMDS,True)
    
    for hbalst in eachHbaModelFwLst:
        hbaObjLst.append(getSingleHbaInfo(hbalst))

    setHbaDriverStateInfo(hbaObjLst,eachHbaDriStateLst)        
    return hbaObjLst
       
       
def setHbaDriverStateInfo(hbaObjLst,driStateLst):
    for hbaObj in hbaObjLst:
        arg = hbaObj.getHbaArg()
        
        driStateLineLst = []
        for lst in driStateLst:
            if is_belong_lst(arg,lst):
                driStateLineLst = lst
                break
        for dLine in driStateLineLst:
            if ('Driver'.lower() in dLine.lower()) and ('state' in dLine.lower()):
                if '=' in dLine:
                    state = re.split('=',dLine)[-1].strip()
                    hbaObj.setState(state)
                continue
            else:
                Logger.info('state not found,Driver state not found in cli' )
            
            
            if ('Driver'.lower() in dLine.lower()) and ('Version'.lower() in dLine.lower()):
                drLst = re.split('\s+',dLine)
                for idx,drValue in enumerate(drLst):
                    try:
                        if 'fcd' in drValue.lower():
                            drVersion = drLst[idx+1]
                            hbaObj.setDrVersion(drVersion)
                    except Exception,e:
                        Logger.info('hba drversion not found.',str(e))
                

def getSingleHbaInfo(hbaLst):
    hbaObj = HbaEntity()
    for line in hbaLst:
        if HBA_MODEL_FW_CMDS in line:
            hhaArg = line.replace(HBA_MODEL_FW_CMDS,'').strip()
            hbaObj.setHbaArg(hhaArg)
        
        if ('Part number'.lower()  in line.lower()) and (':' in line):
            model = ''
            lst = re.split(':',line)
            try:
                model = re.split('\s+',lst[-1].strip())[0]
                hbaObj.setHbaModel(model)
            except Exception,e:
                Logger.info('hba model not found.',str(e))
            continue
        else:
            Logger.info('Part number not found. ROM Firmware version not found.')
        if ('ROM Firmware version'.lower() in line.lower()) and (':' in line):
            fwVersion = ''
            try:
                fwVersion = re.split(':',line)[-1].strip()
                hbaObj.setFwVersion(fwVersion)
            except Exception,e:
                Logger.info('fwVersion not found.',str(e))
        else:
            Logger.info('fwVersion not found. ROM Firmware version not found.')
    return hbaObj
                    
        
    
class HbaEntity:
    hbaArg = ''
    hbaModel = 'NA'
    wwpn = 'NA'
    fwVersion = 'NA'
    drVersion = 'NA'
    state = 'NA'
    
    def getHbaArg(self):
        return self.hbaArg 
    def getHbaModel(self):
        return self.hbaModel
    def getHbaWWPN(self):
        return self.wwpn
    def getFwVersion(self):
        return self.fwVersion
    def getDrVersion(self):
        return self.drVersion
    def getState(self):
        return self.state

    def setHbaModel(self,model):
        self.hbaModel = model
    def setHbaWWPN(self,wwpn):
        self.wwpn = wwpn
    def setFwVersion(self,version):
        self.fwVersion = version
    def setDrVersion(self,drVersion):
        self.drVersion = drVersion
    def setState(self,state):
        self.state = state
    def setHbaArg(self,arg):
        self.hbaArg = arg
        
        
