# coding=utf-8

from com.huawei.ism.tool.infograb.context import EvalResultEnum
from com.huawei.ism.tool.infograb.context import ItemEvalResult
from com.huawei.ism.tool.infograb.entity import Item

import re
import sys
import traceback
import os
path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "..\\..")
sys.path.append(path)
from common import util
from common import contentParse
from common import constants

def execute(context):
    """
    Function name      : execute
    Function describe  : 外部接入
    Input              : context
    Return             : cmd display{}
    cmd_display.put("cmd_display"+(cmd_info_id[rg])[8:], cmd_display_temp)
    cmd_display.put("err_msg", fun_err_msg  + cmd_display.get("err_msg"))
    cmd_display.put("evl_ret", CHECK_FAIL)
    """
    global CLI
    CLI = context.get("SSH")
    global LANGUAGE
    LANGUAGE = context.get("lang")
    cmd_display = context.get("ret_map")
    (cmd_storage_info_id, cmd_storage_info_desc) = confirmCmd()
    check_and_get_result(cmd_storage_execute(cmd_display, CLI, LANGUAGE, cmd_storage_info_id, cmd_storage_info_desc))
    check_and_get_result(cmd_execute(cmd_display, CLI, LANGUAGE))

    return doEvaluate(context)

def doEvaluate(context):
    '''
    @summary: Evaluate ultrapath.
    @param context: Python execution context.
    @return: retDict from python execution context.
    '''
    retDict = context.get("ret_map")
    isQrySucc, ultrapathVersion, cliRet = util.qryUltrapathVersion(context)
    if not isQrySucc:
        util.log.error(context, 'Query ultrapath version failed.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet,
                                  constants.ERR_MSG_KEY_QRY_MULTIPATH_FAILED)
        retDict['evalResult'] = itemEvalResult
        return retDict
    else:
        util.log.debug(context, 'Ultrapath version is:' + unicode(ultrapathVersion))

    isQrySucc, isMultipathdRunning, pathdCliRet = util.qryMultipathdStatus(context)
    contentParse.setCmdRet4Report(context, pathdCliRet, "cmd_display_multipathd_status")
    cliRet += pathdCliRet
    if not isQrySucc:
        util.log.error(context, 'Query multipathd status failed.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet,
                                  constants.ERR_MSG_KEY_QRY_MULTIPATHD_FAILED)
        retDict['evalResult'] = itemEvalResult
        return retDict

    #None-root user has no privilege to query multipathd running status on Linux of some release version.
    if 'may require superuser privileges' in pathdCliRet:
        util.log.error(context, 'No privilege to query multipathd status.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_MULTIPATHD_NO_PRIVILEGE
        suggestMsgKey = constants.SUGGEST_MSG_KEY_QRY_MULTIPATHD_USE_ROOT
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet, errMsgKey)
        itemEvalResult.setSuggestMsgKey(suggestMsgKey)
        return retDict

    if ultrapathVersion and isMultipathdRunning:
        util.log.error(context, 'Ultrapath and multipathd service are running at the same time.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                  constants.ERR_MSG_KEY_ULTRAPATH_MULTIPATHD_ALL_RUNNING)
        retDict['evalResult'] = itemEvalResult
        return retDict

    if not ultrapathVersion and not isMultipathdRunning:
        errMsgKey = constants.ERR_MSG_KEY_ULTRAPATH_MULTIPATHD_ALL_NOT_RUNING
        util.log.error(context, 'Neither Ultrapath nor multipathd service is running.')
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet, errMsgKey)
        retDict['evalResult'] = itemEvalResult
        return retDict

    if ultrapathVersion:#For HUAWEI Ultrapath check.
        itemEvalResult = checkRedundantLinkByUltrapath(context, ultrapathVersion)
        itemEvalResult.setCliRet(cliRet + itemEvalResult.getCliRet())
        retDict['evalResult'] = itemEvalResult
        return itemEvalResult
    else:#For Linux multhpathd check.
        itemEvalResult = checkRedundantLinkByMultipathd(context)
        itemEvalResult.setCliRet(cliRet + itemEvalResult.getCliRet())
        retDict['evalResult'] = itemEvalResult
        return itemEvalResult

def checkRedundantLinkByUltrapath(context, ultrapathVersion):
    '''
    @summary: Check all LUN redundant link with ultrapath.
    @param context: Python execution context.
    @param ultrapathVersion: Ultrapath version
    @return:  item evaluate result of Java bean.
    '''
    tupVer = util.getUltrapathIntVer(context, ultrapathVersion)
    lowerTupVer = util.getUltrapathIntVer(context, constants.LINUX_LOWER_ULTRAPATH_VER)
    middleTupVer = util.getUltrapathIntVer(context, constants.LINUX_MIDDLE_ULTRAPATH_VER)
    if tupVer <= lowerTupVer:
        return checkLowerVerRedundantPath(context)
    elif tupVer <= middleTupVer:
        return checkVlunRedundantLink(context, True)
    else:
        return checkVlunRedundantLink(context, False)

def checkLowerVerRedundantPath(context):
    '''
    @summary: Check all LUN redundant link with ultrapath with version lower than or equal to 5.01.017.
    @param context: Python execution context.
    @return: object of ItemEvalResult 
    '''
    isQrySucc, arrayIDList, cliRet = util.getArrayIdsManagedByUltrapath(context)
    if not isQrySucc:
        util.log.error(context, 'Query array managed by ultrapath failed.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_ARRAY_ID_FAILED

    if not arrayIDList:
        util.log.error(context, 'Query no array managed by ultrapath.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_ARRAY_ID_EMPTY

    if not isQrySucc or not arrayIDList:
        itemEvalResult = genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet, errMsgKey)
        return itemEvalResult

    cli = context.get("SSH")

    noRedundantPathLunIds = []
    for arrayId in arrayIDList:
        try:
            arrayLunRet = cli.execCmdWithNoCheckResult('upadm show lun array=' + arrayId,
                                           constants.HOST_CMD_TIMEOUT,
                                           constants.LINUX_CMD_DFT_END_LIST)
            if re.search("Can't find the vlun", arrayLunRet, re.I):
                util.log.error(context, 'No lun found for array:' + unicode(arrayId))
                return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                                                 cliRet,
                                                 constants.ERR_MSG_KEY_QRY_VLUN_ID_EMPTY)

            cliRet += arrayLunRet
            lunPathDict = parseAllLunPathOfOneArray(arrayLunRet)
            noRedundantPathLunIds.extend(chkAllLunRedundantPath(context, lunPathDict))
        except:
            util.log.error(context, 'Query array IDs exception:' + traceback.format_exc())
            return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL, cliRet,
                                             constants.ERR_MSG_KEY_QRY_ARRAY_LUN_PATH_FAILED)

    if not noRedundantPathLunIds:
        return genEvalResultForMultipath(EvalResultEnum.PASSED, cliRet, '')
    else:
        errParas = [str(cli.getHost())]
        errParas.append(','.join(noRedundantPathLunIds))
        return genEvalResultForMultipath(EvalResultEnum.FAILED,
                         cliRet,
                        constants.ULTRA_NOT_PASS_TIPS,
                        errParas)

def parseAllLunPathOfOneArray(arrayLunRet):
    '''
    @summary: Parse all LUN path of one array according the upadm show lun array = arrayId command result.
    @param arrayLunRet: command result.
    @return: LUN PATH dictionary.
    '''
    lunPathDict = {}
    keyWord = 'Information of Lun'
    if 'Lun Information' in arrayLunRet:
        keyWord = 'Lun Information'
    keyWordLen = len(keyWord)
    lunInfoStartIdx = arrayLunRet.find(keyWord)
    newPathPattern = re.compile(constants.NEW_PATH_FOUND_NEW_PATTERN, re.I)
    pathPattern = re.compile(constants.CTRL_ID_NEW_RE_PATTERN, re.I)
    totLen = len(arrayLunRet)

    while -1 != lunInfoStartIdx and lunInfoStartIdx < totLen - 1:
        nextLunInfoStartIdx = arrayLunRet.find(keyWord, lunInfoStartIdx + keyWordLen)
        lunPath, lunId = '', ''
        for line in arrayLunRet[lunInfoStartIdx:nextLunInfoStartIdx].splitlines():
            line = line.strip().lower()
            if not isValidLunPathInfoLine(line):
                continue

            #find a new LUN
            if 'lun #' in line:
                lunId = line.split('-')[0].split('lun #')[-1].strip()
                lunPathDict[lunId] = []
                continue
            elif 'information of lun' in line:
                lunId = line.split('-')[0].split('information of lun')[-1].strip()
                lunPathDict[lunId] = []
                continue

            #new lun path found
            if newPathPattern.search(line) and pathPattern.search(line):
                lunPath = pathPattern.search(line).group()[1:]
                continue

            if lunPath and line.startswith('numlunobjects'):
                objNum = int(line.split(':')[1].split()[0].strip())
                if objNum > 0:
                    pass
                else:
                    lunPath = ''
                continue

            if lunPath and 'devstate' in line and len(line.split(':')) <= 2:
                devState = line.split(':')[-1].strip()
                if 'optimal' == devState.lower():
                    if lunId in lunPathDict:
                        lunPathDict[lunId].append(lunPath)

        lunInfoStartIdx = nextLunInfoStartIdx
    return lunPathDict

def isValidLunPathInfoLine(line):
    lowerLine = line.lower()
    return ('lun name' in lowerLine or 'controller' in line
         or 'numlunobjects' in line or ('devstate' in line and 'bootowningpath' not in line))

def chkAllLunRedundantPath(context, lunPathDict):
    noRedundantPathLunIds = []
    for lunId in lunPathDict:
        lunPath = lunPathDict.get(lunId)
        if not util.hasRedundantPathForCtrlIdPath(context, lunPath):
            noRedundantPathLunIds.append(lunId)
    return noRedundantPathLunIds

def checkVlunRedundantLink(context, isMiddleVer = True):
    '''
    @summary: Check all LUN redundant link with ultrapath with version higher than 5.01.017
    @param context: Python execution context.
    @return: object of ItemEvalResult 
    '''
    isQrySucc, vlunIdList, cliRet = util.getVlunIdsManagedByUltrapath(context, isMiddleVer)
    if not isQrySucc:
        util.log.error(context, 'Query vluns managed by ultrapath failed.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_VLUN_ID_FAILED

    if not vlunIdList:
        util.log.error(context, 'Query no array managed by ultrapath.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_VLUN_ID_EMPTY

    if not isQrySucc or not vlunIdList:
        return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                                  cliRet, errMsgKey)

    cli = context.get("SSH")
    noRedundantPathLunIdList = []
    hyperMetroLunNoRedundantIdList = []
    for vlunId in vlunIdList:
        try:
            qryVlunPthCmd = 'upadmin show vlun id=' + vlunId + ('' if isMiddleVer else ' type=all')
            vlunPthRet = cli.execCmdWithNoCheckResult(qryVlunPthCmd,
                                                      constants.HOST_CMD_TIMEOUT,
                                                      constants.LINUX_CMD_DFT_END_LIST)
            cliRet += vlunPthRet
            isHyperMetroLUN, chkPass = util.chkSingleLunPathRedundant(context, vlunPthRet)

            if not isHyperMetroLUN:
                if not chkPass:
                    util.log.error(context, 'Normal Vlun path not redundant:' + vlunId)
                    noRedundantPathLunIdList.append(vlunId)
                else:
                    util.log.debug(context, 'Normal or migration Vlun path redundant:' + vlunId)
            elif chkPass:
                util.log.debug(context, 'Hypermetro Vlun path redundant:' + vlunId)
            else:
                hyperMetroLunNoRedundantIdList.append(vlunId)
                util.log.error(context,
                               'Hypermetro Vlun path not redundant '
                               'or from different array or missing sub LUN:' + vlunId)
        except:
            util.log.error(context, 'Query VLUN path exception:' + traceback.format_exc())
            return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                                  cliRet, constants.ERR_MSG_KEY_QRY_SINGLE_LUN_PATH_FAILED)

    if not noRedundantPathLunIdList and not hyperMetroLunNoRedundantIdList:
        return genEvalResultForMultipath(EvalResultEnum.PASSED, cliRet, '')
    else:
        errParas = [str(cli.getHost())]
        if noRedundantPathLunIdList and not hyperMetroLunNoRedundantIdList:
            errParas.append(','.join(noRedundantPathLunIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                         constants.ULTRA_NOT_PASS_TIPS,
                                         errParas)
        elif not noRedundantPathLunIdList and hyperMetroLunNoRedundantIdList:
            errParas.append(','.join(hyperMetroLunNoRedundantIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                             constants.ULTRA_HYPER_METRO_VLUN_NOT_PASS_TIPS,
                                             errParas)
        else:
            errParas.append(','.join(noRedundantPathLunIdList))
            errParas.append(','.join(hyperMetroLunNoRedundantIdList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED, cliRet,
                                             constants.ULTRA_VLUN_NOT_PASS_TIPS,
                                             errParas)

def checkRedundantLinkByMultipathd(context):
    '''
    @summary: 评估自带多路径管理的存储LUN是否存在冗余路径。
    @param context: 上下文
    @return:评估结果.
    '''
    isQrySucc, portNumbers, cliRet = util.getTargetPortNum(context)
    if not isQrySucc:
        util.log.error(context, 'Query target port number reported by storage failed.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_TGT_PORT_NUM_FAILED

    if not portNumbers:
        util.log.error(context, 'Query no array managed by ultrapath.')
        errMsgKey = constants.ERR_MSG_KEY_QRY_TGT_PORT_NUM_EMPTY

    if not isQrySucc or not portNumbers:
        return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                                  cliRet, errMsgKey)

    util.log.debug(context, 'Target port number list:' + unicode(portNumbers))
    tgtPortNumPthMap, qryPthCliRet = util.getPortNumPathDict(context, portNumbers)
    cliRet += ('\n' + qryPthCliRet)
    if not tgtPortNumPthMap:
        errMsgKey = constants.ERR_MSG_KEY_QRY_TGT_PORT_PATH_EMPTY
        util.log.error(context, 'Query all the path of target port failed.')
        return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                                  cliRet, errMsgKey)

    util.log.debug(context, 'Target port number path:' + unicode(tgtPortNumPthMap))
    return evaluteAllDiskManagedByMultiPathd(context, tgtPortNumPthMap)

def evaluteAllDiskManagedByMultiPathd(context, tgtPortNumPthMap):
    '''
    @summary: 根据查询到的端口路径映射关系，判断自带多路径服务multipathd管理的所有华为磁盘是否存在冗余链路。
    @param context: Python执行上下文.
    @param tgtPortNumPthMap: TGT端口和路径的映射字典。
    @return: isEvalSuccess, notRedundantDiskList, cliRet
    '''
    cli = context.get("SSH")
    notRedundantPathDiskList = []
    allPthRet = ''
    try:
        qryAllDiskPthCmd = constants.QRY_DISK_PTH_MANAGED_BY_MULTIPATHD_CMD
        allPthRet = cli.execCmdWithNoCheckResult(qryAllDiskPthCmd,
                                       constants.HOST_CMD_TIMEOUT,
                                       constants.LINUX_CMD_DFT_END_LIST)

        diskPathDict = parseAllDiskPth(allPthRet, tgtPortNumPthMap)
        if not diskPathDict:#no disk found, unable to evaluate.
            return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                             allPthRet,
                            constants.ERR_MSG_MULTIPATHD_NO_HW_DISK_FOUND)

        for diskName in diskPathDict:
            util.log.debug(context, 'LunName:' + unicode(diskName)
                           + 'LunPath:' + unicode(diskPathDict.get(diskName)))
            if not util.hasRedundantPathForCtrlIdPath(context, diskPathDict.get(diskName), isNodeId = True):
                notRedundantPathDiskList.append(diskName)

        if not notRedundantPathDiskList:
            return genEvalResultForMultipath(EvalResultEnum.PASSED, allPthRet, '')
        else:
            errParas = [str(cli.getHost())]
            errParas.append(','.join(notRedundantPathDiskList))
            return genEvalResultForMultipath(EvalResultEnum.FAILED,
                             allPthRet,
                            constants.NMP_NOT_PASS_TIPS,
                            errParas)
    except:
        util.log.error(context, 'Query disk path exception:' + traceback.format_exc())
        return genEvalResultForMultipath(EvalResultEnum.UNABLE_EVAL,
                              allPthRet, constants.ERR_MSG_KEY_QRY_SINGLE_LUN_PATH_FAILED)

def parseAllDiskPth(allPthRet, tgtPortNumPthMap):
    '''
    @summary: 解析multipaht - ll命令回文，生成磁盘名称到磁盘路径的字典。
    @param allPthRet: multipaht - ll回文。
    @param tgtPortNumPthMap: TGT端口到路径的映射字典，如：{'0:0:0':'0A', '0:0:1':'0b'}。
    @return: 磁盘名称到磁盘路径的字典。如：{'dm-2': ['0c', '0d'], 'dm-11': ['0c', '0b']}
    '''
    tgtPortNumPattern = re.compile(constants.TARGET_PORT_NUM_PATTERN, re.I)
    newDiskPattern = re.compile(constants.NEW_HW_DISK_PATTERN, re.I)
    aboveStaPattern = re.compile('.+prio.+(active|enabled)', re.I)
    belowStaPattern = re.compile('ready', re.I)
    diskPathDict = {}
    lines = allPthRet.splitlines()
    maxIdx = len(lines)
    i = 0
    while i < maxIdx:
        diskNameMatchObj = newDiskPattern.search(lines[i])
        if diskNameMatchObj:
            diskName = diskNameMatchObj.group().split()[0].strip()
            j = i + 2
            while not newDiskPattern.search(lines[j]) and j + 1 < maxIdx:
                aboveStaMatchObj = aboveStaPattern.search(lines[j])
                if aboveStaMatchObj:#找到第一个状态值后，遍历查找其下的所有正常路径。
                    pthIdx = j + 1
                    tgtPortNumMatchObj = tgtPortNumPattern.search(lines[pthIdx])
                    while tgtPortNumMatchObj:#匹配到TGT端口号.
                        pathStaMatchObj = belowStaPattern.search(lines[pthIdx])
                        if pathStaMatchObj:#状态正常
                            tgtPortNum = tgtPortNumMatchObj.group()
                            path = tgtPortNumPthMap.get(tgtPortNum)
                            if diskName in diskPathDict:
                                diskPathDict.get(diskName).append(path)
                            else:
                                diskPathDict[diskName] = [path]
                        #Find next path in next line.
                        pthIdx += 1
                        tgtPortNumMatchObj = tgtPortNumPattern.search(lines[pthIdx])
                    else:#非路径行，下一行可能是状态行或者新的磁盘信息。
                        j = pthIdx
                else:
                    j += 1
            else:#Inner while loop end, to find a new disk.
                i = j
        else:
            i += 1
    return diskPathDict

def genEvalResultForMultipath(evalResult, cliRet, errMsgKey, paraList = None):
    '''
    @summary: Create an evaluate result object of Java ItemEvalResult type.
    @param evalResut: Evaluate result.
    @param errMsgKey: error message key
    @param cliRet: CLI return message.
    @param paraList:Error message parameter list of string type.
    @return: item evaluate result.
    '''
    itemEvalResult = util.genEvalItemObj(Item.MULTIPATH_CHECK_ID,
                                         evalResult,
                                         cliRet, errMsgKey, paraList)
    return itemEvalResult


CLI = None
LANGUAGE = None
CHECK_FAIL = 0
CHECK_PASS = 1
cmd_info_id = ["cmd_info_multipath_device_mapper"  ,
               "cmd_info_multipath_multipath"      ,
               "cmd_info_multipath_powermt_version",
               "cmd_info_multipath_powermt_display",
               "cmd_info_multipath_config"]

cmd_info_desc = ["rpm -qa |grep device-mapper",
                 "multipath -ll",
                 "powermt version",
                 "powermt display dev=all",
                 "cat /etc/multipath.conf"]

cmd_multipath_Huawei_id = ['cmd_info_multipath_version_huawei', 'cmd_info_multipath_path_huawei']
cmd_multipath_Huawei_desc = ['upadmin show version', 'upadmin show vlun']
cmd_multipath_Huawei = zip(cmd_multipath_Huawei_id, cmd_multipath_Huawei_desc)

cmd_multipath_Huawei_old_id = ['cmd_info_multipath_version_huawei_old', 'cmd_info_multipath_path_huawei_old']
cmd_multipath_Huawei_old_desc = ['upadm show version', 'upadm show vlun']
cmd_multipath_Huawei_old = zip(cmd_multipath_Huawei_old_id, cmd_multipath_Huawei_old_desc)

cmd_multipath_EMC_id = ['cmd_info_multipath_version_emc', 'cmd_info_multipath_path_emc']
cmd_multipath_EMC_desc = ['powermt version', 'powermt display dev=all']
cmd_multipath_EMC = zip(cmd_multipath_EMC_id, cmd_multipath_EMC_desc)

cmd_multipath_IBM_id = ['cmd_info_multipath_version_ibm', 'cmd_info_multipath_path_ibm']
cmd_multipath_IBM_desc = ['lslpp -L |grep -i sddpcm', 'pcmpath query device']
cmd_multipath_IBM = zip(cmd_multipath_IBM_id, cmd_multipath_IBM_desc)

cmd_multipath_HDS_id = ['cmd_info_multipath_sys_hds', 'cmd_info_multipath_lu_hds', 'cmd_info_multipath_path_hds']
cmd_multipath_HDS_desc = ['dlnkmgr view -sys', 'dlnkmgr view  -lu', 'dlnkmgr view -path']
cmd_multipath_HDS = zip(cmd_multipath_HDS_id, cmd_multipath_HDS_desc)

cmd_multipath_IBMSDDDSM_id = ['cmd_info_multipath_ibmsdddatapath']
cmd_multipath_IBMSDDDSM_desc = ['rpm -qa|grep -i IBMsdd or /opt/IBMsdd/bin/datapath query version']
cmd_multipath_IBMSDDDSM = zip(cmd_multipath_IBMSDDDSM_id, cmd_multipath_IBMSDDDSM_desc)

cmd_multipath = [cmd_multipath_Huawei,
                 cmd_multipath_Huawei_old,
                 cmd_multipath_EMC,
                 cmd_multipath_IBM,
                 cmd_multipath_HDS,
                 cmd_multipath_IBMSDDDSM
                 ]

#确认执行命令
def confirmCmd():
    cmdPos = []
    succCmd_id = []
    succCmd_desc = []
    index = -1
    for cmd in cmd_multipath :
        index = index + 1
        for cmd_id, cmd_desc in cmd:
            succ = execTrail(cmd_desc)
            if succ:
                if not index in cmdPos:
                    cmdPos.append(index)
    if len(cmdPos) == 0:
        cmdPos.append(-1)
    for pos in cmdPos:
        if pos == -1:
            for cmd in cmd_multipath :
                for cmd_id, cmd_desc in cmd:
                    succCmd_id.append(cmd_id)
                    succCmd_desc.append(cmd_desc)
        else:
            cmd = cmd_multipath[pos]
            for cmd_id, cmd_desc in cmd:
                succCmd_id.append(cmd_id)
                succCmd_desc.append(cmd_desc)
    return succCmd_id, succCmd_desc

#尝试执行命令
def execTrail(cmd):
    cmd_display_temp = CLI.execCmdNoLog(cmd)
    if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
        return False
    else:
        return True


# 取中间一个有效值
def check_and_get_result((fun_flag, result, fun_err_msg)):
    """
    :rtype : object
    """
    return result

# 取一个有效值
def check_storage_and_get_result((fun_flag, result, fun_err_msg)):
    """
    :rtype : object
    """
    return result

def cmd_storage_execute(cmd_display, CLI, LANGUAGE, cmd_info_id, cmd_info_desc):
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    for rg in range(len(cmd_info_id)):
        cmd_display_temp = CLI.execCmdNoLogTimout(cmd_info_desc[rg], 60)
        cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:], cmd_display_temp)
        if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute failed\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行失败\r\n"
        else:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute success\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行成功\r\n"
    cmd_display.put("err_msg", fun_err_msg);
    fun_flag = CHECK_PASS
    return fun_flag, cmd_display, fun_err_msg


def cmd_execute(cmd_display, CLI, LANGUAGE):
    fun_flag = CHECK_FAIL
    fun_err_msg = ''
    for rg in range(len(cmd_info_id)):
        cmd_display_temp = CLI.execCmdNoLogTimout(cmd_info_desc[rg], 60)
        cmd_display.put("cmd_display" + (cmd_info_id[rg])[8:], cmd_display_temp)
        if None == cmd_display_temp or '' == cmd_display_temp or cmd_display_temp.find('TOOLKIT_SEND_CMD_TIME_OUT') > 0 or cmd_display_temp.find('TOOLKIT_EXE_CMD_FAILED') > 0:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute failed\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行失败\r\n"
        else:
            if "en" == LANGUAGE:
                fun_err_msg += cmd_info_desc[rg] + ":\texecute success\r\n"
            else:
                fun_err_msg += cmd_info_desc[rg] + u":\t执行成功\r\n"
    cmd_display.put("err_msg", fun_err_msg + cmd_display.get("err_msg"))
    fun_flag = CHECK_PASS
    return fun_flag, cmd_display, fun_err_msg
