# coding=utf-8
from com.huawei.ism.tool.infograb.context import EvalResultEnum

import re
import sys
import os

path = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(path, "..")
sys.path.append(path)
from common import util
from common import constants
from common import contentParse

cmdInfoIdList = ["cmd_info_os_for_windows", "cmd_info_os_for_unix", "cmd_info_multipath_version_new", "cmd_info_multipath_version_old"]
commandList = ["echo %OS%", "uname -a", "upadmin show version", "upadm show version"]
osTypeFor501063List = [constants.OS_TYPE_LINUX, constants.OS_TYPE_SOLARIS, constants.OS_TYPE_WINDOWS]


def execute(context):
    '''
    Function name      : execute
    Function describe  : 外部接入
    Input              : context
    '''
    checkUltrapathVersion(context)

def checkUltrapathVersion(context):
    '''
    @summary 评估，并返回评估结果
    @param context : 上下文
    '''
    intSoftVersion501022 = util.getUltrapathIntVer(context, "5.01.022")
    intSoftVersion501063 = util.getUltrapathIntVer(context, "5.01.063")
    intSoftVersion601007 = util.getUltrapathIntVer(context, "6.01.007")
    intSoftVersion601025 = util.getUltrapathIntVer(context, "6.01.025")
    retDict = context.get("ret_map")

    itemKey = "DIFNotIdentityCheck"
    evalResult = EvalResultEnum.PASSED
    cliRet = ""
    errMsgKey = ""

    osType, tempCliRet = getOSType(context)
    cliRet += tempCliRet + "\r\n"
    if "NA" == osType:
        evalResult = EvalResultEnum.UNABLE_EVAL
        errMsgKey = constants.ERR_MSG_DIF_NOT_IDENTITY_UNABLE_EVAL
        itemEvalResult = util.genEvalItemObj(itemKey, evalResult, cliRet, errMsgKey)
        retDict["evalResult"] = itemEvalResult
    else:
        softVer, tempCliRet = getUltrapathVersion(osType, context)
        util.log.info(context, "The version is :" + softVer)
        cliRet += tempCliRet + "\r\n"

        if "Pass" == softVer:
            itemEvalResult = util.genEvalItemObj(itemKey, evalResult, cliRet, errMsgKey)
            retDict["evalResult"] = itemEvalResult
        #无法评估
        elif "NA" == softVer:
            evalResult = EvalResultEnum.UNABLE_EVAL
            errMsgKey = constants.ERR_MSG_DIF_NOT_IDENTITY_UNABLE_EVAL
            itemEvalResult = util.genEvalItemObj(itemKey, evalResult, cliRet, errMsgKey)
            retDict["evalResult"] = itemEvalResult
        else:
            intSoftVersion = util.getUltrapathIntVer(context, softVer)
            util.log.info(context, "Change the version to int is :" + str(intSoftVersion))

            if constants.OS_TYPE_AIX == osType and intSoftVersion501022 <= intSoftVersion <= intSoftVersion601025:
                evalResult = EvalResultEnum.WARNING
                errMsgKey = constants.ERR_MSG_DIF_NOT_IDENTITY_NOT_PASSED
            elif osType in osTypeFor501063List and intSoftVersion501063 <= intSoftVersion <= intSoftVersion601007:
                evalResult = EvalResultEnum.WARNING
                errMsgKey = constants.ERR_MSG_DIF_NOT_IDENTITY_NOT_PASSED
            itemEvalResult = util.genEvalItemObj(itemKey, evalResult, cliRet, errMsgKey)
            retDict["evalResult"] = itemEvalResult

def getUltrapathVersion(osType, context):
    '''
    @summary 评估，并返回评估结果
    @param context : 上下文
    @param osType : 操作系统
    @return: softVer多路径软件版本，cliRet命令回文
    '''
    cliRet = ""
    softVer = "NA"
    if constants.OS_TYPE_LINUX == osType:
        cliRet = contentParse.getSingleCommandRetPureRetWithoutResCheck(context, cmdInfoIdList[2], commandList[2])
    else:
        cliRet = contentParse.getSingleCommandRetPureRetWithoutResCheck(context, cmdInfoIdList[3], commandList[3])

    if "'upadm'" in cliRet and constants.OS_TYPE_WINDOWS == osType:
        return "Pass", cliRet

    resultList = cliRet.splitlines()
    for line in resultList:
        if ("software" in line.lower() and "version" in line.lower()) or "ultrapath for" in line.lower():
            softVer = re.split(":", line)[-1].strip()
            break
        #当前未安装自验多路径直接通过，不考虑Windows
        elif "not found" in line.lower():
            softVer = "Pass"
            break
    return softVer, cliRet

def getOSType(context):
    '''
    @summary 评估，并返回评估结果
    @param context : 上下文
    @return: softVer多路径软件版本，cliRet命令回文
    '''

    """先判断是否是Windows"""
    cliRet = contentParse.getSingleCommandRetPureRetWithoutResCheck(context, cmdInfoIdList[0], commandList[0])
    if constants.OS_TYPE_WINDOWS.lower() in cliRet.lower():
        return constants.OS_TYPE_WINDOWS, cliRet

    """再判断其他系统"""
    cliRet = contentParse.getSingleCommandRetPureRetWithoutResCheck(context, cmdInfoIdList[1], commandList[1])
    if constants.OS_TYPE_LINUX.lower() in cliRet.lower():
        return constants.OS_TYPE_LINUX, cliRet

    if constants.OS_TYPE_SOLARIS.lower() in cliRet.lower() or "SunOS".lower() in cliRet.lower():
        return constants.OS_TYPE_SOLARIS, cliRet

    if constants.OS_TYPE_AIX.lower() in cliRet.lower():
        return constants.OS_TYPE_AIX, cliRet

    return "NA" , cliRet

